c -------------------------------------------------------------------- c
c -----  This file contains the constraints to inverse seesaw    ----- c
c -----  mechanisms taken from experiments.                      ----- c
c -------------------------------------------------------------------- c


************************************************************************
************************************************************************
*This file is a part of NMSSMCALC-nuSS
*This file contains:
*
*  subroutine IssConstraints: check all five constraints
*
*  SUBROUTINE IssNeuDataConstraints
*       Constraints from neutrino oscillation data
*
*  SUBROUTINE IssMSumConstraints
*       Constraints on sum of neutrino masses from cosmological Planck
*       experiment.
*
*  SUBROUTINE IssNoUConstraints
*       Constraints on the unitarity of neutrino mixing matrix taken
*       from Electroweak Precision Observables (EWPO)
*
*  SUBROUTINE IssLfvConstrains
*       Constraints on lepton flavour violation
*
*  real*16 function Lep2LepGamDecay
*       compute branching ratios of the l1->l2 +gamma decays
* 
* subroutine Yvoutput: to write output of neutrino Yukawa in bhdecay_c.f
************************************************************************
************************************************************************

#include "def.h"

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine IssConstraints(isscheck)
      
      implicit none
      integer icheck,id
      logical isscheck(10)
      double precision output(3),brcpn(3,6,2)
#include "issnmssm.h"
      
c --- Preliminary procedure ---
      icheck = 0
      isscheck(:) = .true.  ! isscheck is true if constraint satisfied
      
c --- Check for constraints on neutrino oscillation ---
      call IssNeuDataConstraints(UPMNS,mv,icheck)
      if (icheck .eq. 1) then
        isscheck(1) = .false.
        icheck = 0
      endif
      
c --- Check for neutrino sum constraints ---
      call IssMSumConstraints(mv9,icheck)
      if (icheck .eq. 2) then
        isscheck(2) = .false.
        icheck = 0
      endif
        
c --- Check for STU parameter ---
      call issSTU_check(icheck)
      if (icheck .eq. 3) then
        isscheck(3) = .false.
        icheck = 0
      endif

        
c --- Check for lepton number violation of mixing matrix ---
      call IssLfvConstraints(icheck)
      if (icheck .eq. 4) then
        isscheck(4) = .false.
        icheck = 0
      endif
      

c --- Check for unitarity of mixing matrix ---
      call IssNoUConstraints(UV, UPMNS,mv9,icheck)
      if (icheck .eq. 5) then
        isscheck(5) = .false.
        icheck = 0
      endif
       print*, "isscheck(1:5)",isscheck(1:5)
      return
      end subroutine IssConstraints



c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine IssNeuDataConstraints(UVinp,mv,icheck)
      implicit none
      integer icheck
c       icheck == 1 means neutrino oscillation constraint violated
c       icheck == 0 means satisfied
      real*16 mv(3),deltam31,deltam21,deltam32,sin12,sin23,sin13,ev
      complex*32 UVinp(3,3), UV(3,3)
      parameter (ev=1D-18) 
      logical iNH
c       iNH is used to determine the hierarchy of neutrino
c       iNH = true  if m1 < m2 < m3 i.e. normal hierarchy
c       iNH = false if m3 < m1 < m2 i.e. inverted hierarchy

c       Check for normal or inverted hierarchy
        if (mv(1) .lt. mv(2) .and. mv(2) .lt. mv(3)) then
          iNH = .True.
        else
          iNH = .False.
        endif
c       UV(1:3,1:3) = transpose(UPMNS) due to
c       interaction --UV--> mass while mass --UPMNS--> interaction
        UV = UVinp

      deltam21= (mv(2)*1d9)**2 - (mv(1)*1d9)**2
      sin12= (UV(1,2)/UV(1,1))**2 / (1+ (UV(1,2)/UV(1,1))**2)
      if (deltam21.lt.6.82D-5  .OR. deltam21.gt.8.04D-5 ) icheck=1  
       print*, icheck  !nhung is checking
      if (sin12 .lt. 0.269 .or. sin12 .gt. 0.343) icheck = 1
      if (iNH) then
       deltam31=(mv(3)*1d9)**2 - (mv(1)*1d9)**2
       if (deltam31.lt.2.435D-3  .OR. deltam31.gt.2.598D-3 ) icheck=1  
       sin23= (UV(2,3)/sqrt(1 -abs(UV(1,3))**2) )**2
       sin13= 1- (UV(1,1)**2 +UV(1,2)**2)
       if (sin23 .lt. 0.415 .or. sin23 .gt. 0.616) icheck = 1
       if (sin13 .lt. 0.02032 .or. sin13 .gt. 0.0241) icheck = 1
      else
       deltam32=(mv(2)*1d9)**2 -(mv(3)*1d9)**2
       if (deltam32.lt.2.414D-3  .OR. deltam32.gt.2.581D-3 ) icheck=1  
       sin23= 1- (UV(3,3)/(sqrt(1 -abs(UV(1,3))**2) ))**2
       sin13= 1- (UV(1,1)**2 +UV(1,2)**2)
       if (sin23 .lt. 0.419 .or. sin23 .gt. 0.617) icheck = 1
       if (sin13 .lt. 0.02052 .or. sin13 .gt. 0.02428) icheck = 1     
      endif

c data here are taken from https://arxiv.org/abs/2007.14792
C       print*, "test new neutrino oscilation data"
C       print*, "delta m12^2:    ",deltam21,
C      .        ", expbestfit=7.42 1D-5 eV^2 +-3sigma(6.82-8.04)"   
C       print*, "delta m31^2:    ",deltam31,
C      .        ", expbestfit= 2.517 1D-3 eV^2 +-3sigma(2.435-2.598)"
C       print*, "delta m32^2:    ",deltam32, ! inverted hierarchy
C      .        ", expbestfit= -2.498 1D-3 eV^2 +-3sigma(2.414-2.581)"
C       print*, "Sin(theta12)^2: ",sin12,
C      .        ", expbestfit= 0.304  +-3sigma(0.269-0.343)"
C       print*, "Sin(theta23)^2: ",sin23,
C      .        ", expbestfit= 0.573  +-3sigma(0.415-0.616), delta31>0"
C       print*, "Sin(theta23)^2: ",sin23,
C      .        ", expbestfit= 0.575  +-3sigma(0.419-0.617), delta32<0"
C       print*, "Sin(theta13)^2: ",sin13,
C      .        ", expbestfit= 0.02219  +-3sigma(0.02032-0.02410), delta31>0"
C       print*, "Sin(theta13)^2: ",sin13,
C      .        ", expbestfit= 0.02238  +-3sigma(0.02052-0.02428), delta32<0"
      end subroutine IssNeuDataConstraints
      
      
      
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine IssMSumConstraints(mv,icheck)
      implicit none
      integer icheck
c       icheck == 2 means neutrino oscillation constraint violated
c       icheck == 0 means satisfied
      real*16 mv(9),msum
      msum = mv(1)+mv(2)+mv(3)
      if (msum .gt. 0.12D-9 ) icheck =2
c msum < 0.12 ev data taken from e-Print:1807.06209, Planck 2018 results. VI. Cosmological parameters
      end subroutine IssMSumConstraints
     
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine IssNoUConstraints(UV, UPMNS,mv,icheck)
      implicit none
      integer i,j,k,icheck
c       icheck == 5 means neutrino oscillation constraint violated
c       icheck == 0 means satisfied
      real*16 mv(9), MI(3,3), tolerance
      complex*32 UV(9,9),UN(3,3), eta(3,3),prod,UPMNS(3,3)
      MI(:,:)=0D0
      MI(1,1)=1D0;MI(2,2)=1D0;MI(3,3)=1D0

      UN(1,1:3) = UV(1,1:3)
      UN(2,1:3) = UV(2,1:3)
      UN(3,1:3) = UV(3,1:3)
      
      do i=1,3
       do j=1,3
c         print*, "UN,UPMNS",UN(i,j),UPMNS(i,j)
        prod=0D0
        do k=1,3
            prod=prod + UN(i,k)* conjg(UN(j,k))
       enddo
        eta(i,j)=MI(i,j)- prod
c        print*, "prod", i,j,  eta(i,j),abs(eta(i,j))
      enddo
      enddo

      if(abs(eta(1,1)) .gt. 0.07 ) icheck =5
      if(abs(eta(2,2)) .gt. 0.07 ) icheck =5
      if(abs(eta(2,1)) .gt. 0.03 ) icheck =5

c the parameterization UN  is taken in the paper
c https://arxiv.org/pdf/1612.07377.pdf,     
c data here are taken from fig.2 and 3 with a parameterization-independent way

      end subroutine ISSNoUconstraints

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine IssNoUConstraintsold(UV, UPMNS,mv,icheck)
      implicit none
      integer i,j,k,icheck
c       icheck == 5 means neutrino oscillation constraint violated
c       icheck == 0 means satisfied
      real*16 mv(9), MI(3,3), tolerance
      complex*32 UV(9,9),UN(3,3), eta(3,3),prod,UPMNS(3,3)
      MI(:,:)=0D0
      MI(1,1)=1D0;MI(2,2)=1D0;MI(3,3)=1D0
      
c nhung changed 4/2021
      UN(1,1:3) = UV(1,1:3)
      UN(2,1:3) = UV(2,1:3)
      UN(3,1:3) = UV(3,1:3)
c end nhung change 4/2021
      
      do i=1,3
       do j=1,3
c         print*, "UN,UPMNS",UN(i,j),UPMNS(i,j)
        prod=0D0
        do k=1,3
            prod=prod + UN(i,k)* conjg(UPMNS(j,k))
       enddo
        eta(i,j)=MI(i,j)- prod
c        print*, "prod", i,j,  eta(i,j),sqrt(2*abs(eta(i,j)))
      enddo
      enddo
c	  stop

      if(sqrt(2*abs(eta(1,1))) .gt. 0.05 ) icheck =5
      if(sqrt(2*abs(eta(2,2))) .gt. 0.021 ) icheck =5
      if(sqrt(2*abs(eta(3,3))) .gt. 0.075 ) icheck =5
      if(sqrt(2*abs(eta(1,2))) .gt. 0.026 ) icheck =5
      if(sqrt(2*abs(eta(1,3))) .gt. 0.052 ) icheck =5
      if(sqrt(2*abs(eta(2,3))) .gt. 0.035 ) icheck =5

c the parameterization UN = (I - eta) UPMNS is taken in the paper
c https://arxiv.org/pdf/1605.08774.pdf , equation 3, page 3      
c data here are taken from https://arxiv.org/pdf/1605.08774.pdf table 4
c the upper limits using two sigma uncertainty for GSS (general see-saw)
c sqrt(2 etaee) <0.05, sqrt(2 etamumu) <0.021, sqrt(2 etatt) <0.075
c sqrt(2 etaemu) <0.026, sqrt(2 etaet) <0.052, sqrt(2 etamut) <0.035
      end subroutine ISSNoUconstraintsold
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine IssLfvConstraints(icheck)
      
      implicit none
      integer icheck
c       icheck == 4 means neutrino oscillation constraint violated
c       icheck == 0 means satisfied
      
      real*16 mu2e, tau2mu, tau2e, Lep2LepGamDecay
      real*16 muDecay, tauDecay
      integer eID, muID, tauID
#include "issnmssm.h"
c --- Define several parameters ---
c     ID for C0i function
      cc0 = 0
      cc1 = 1
      cc2 = 2
      cc00 = 10
      cc11 = 11
      cc12 = 12
      cc22 = 22
    
c     ID for leptons
      eID = 1
      muID = 2
      tauID = 3
c --- Compute branching ratios ---
      mu2e = Lep2LepGamDecay(muID,eID)
      tau2e = Lep2LepGamDecay(tauID,eID)
      tau2mu = Lep2LepGamDecay(tauID,muID)
      brmuega=mu2e;brtauega=tau2e;brtaumuga=tau2mu
c --- Compare with experiment ---
      if (mu2e.gt.4.2D-13 .or. tau2e.gt.3.3D-8 .or.
     -    tau2mu.gt.4.4D-8) then
        icheck = 4
      endif
      
c     Data here are taken from PDG
      end subroutine IssLfvConstraints
      
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      real*16 function Lep2LepGamDecay(init,fin)
      
      implicit none
      integer init,fin,IndexDelta(3,3)
c     init, fin are ID of the initial and final lepton
c     ID = 1 for e, 2 for mu, 3 for tau
      complex*32 FF(2), output
      complex*32 partSMFF(2,3),partHpmFF(2,3),SMFF(2),HpmFF(2),ChaFF(2)
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision ZH5(5,5),ZH(6,6)
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      
      integer i18,i2,j2,i9,i,j
      double complex DB0,SW,CB,IC
      real*16 MassHpm(2), MassFe(3), MVW, MassCha(2),SB2,SB,
     &        C0x,C1x,C2x,C11x,C12x,C22x,x,y,brin
#include "issnmssm.h"
       C0x(x,y)=(-(1./(-1 + y/x)) + Log(y/x)/(-1 + y/x)**2)/x
       C1x(x,y)=((-3. + y/x)/(4.*(-1 + y/x)**2) + Log(y/x)/(2.*(-1. + y/x)**3))/x
       C11x(x,y)=((-11. + (7*y)/x - (2.*y**2)/x**2)/(18.*(-1 + y/x)**3) 
     &            + Log(y/x)/(3.*(-1. + y/x)**4))/x

      MassHpm(1) = MW
      MassHpm(2) = MHpm
      
      MassFe(1) = me
      MassFe(2) = mmu
      MassFe(3) = mtau
      MVW = MW
      SW=Sqrt(1- MW**2/MZ**2)
      
      MassCha(:) = MCha(:)
      SB=sin(beta)
      SB2= SB**2
      CB=Sqrt(1d0-sb2)

      IndexDelta(:,:)=0d0
      IndexDelta(1,1)=1d0
      IndexDelta(2,2)=1d0
      IndexDelta(3,3)=1d0
#ifdef IMPLICIT_OLD
      SMFF(:)=0D0
      do i9 = 1,9  
       SMFF(1)=SMFF(1)+(e*(3*C11x(mv9(i9)**2,MVW**2)+2*C1x(mv9(i9)**2,MVW**2))*
     &  CLFepFvHm(fin,i9,1)*CRFvFemHp(i9,init,1)*MassFe(init))/(16d0*
     &  Pi**2)-(e*(C0x(mv9(i9)**2,MVW**2)+2*C1x(mv9(i9)**2,MVW**2))*
     &  CLFepFvHm(fin,i9,1)*CLFvFemHp(i9,init,1)*mv9(i9))/(8d0*Pi**2)+
     &  (e**3*(CB**2+SB**2)*v*C1x(mv9(i9)**2,MVW**2)*
     &  CLFepFvHm(fin,i9,1)*UV(i9,init))/(16d0*Sqrt2*Pi**2*SW**2)+(e*
     &  MassFe(fin)*(C1x(mv9(i9)**2,MVW**2)*(2*SW**2*
     &  CLFvFemHp(i9,init,1)*CRFepFvHm(fin,i9,1)-e**2*CONJG(UV(i9,fin)
     &  )*UV(i9,init))+3*C11x(mv9(i9)**2,MVW**2)*(SW**2*
     &  CLFvFemHp(i9,init,1)*CRFepFvHm(fin,i9,1)+e**2*CONJG(UV(i9,fin)
     &  )*UV(i9,init))))/(16d0*Pi**2*SW**2)

       SMFF(2)=SMFF(2)+(e**3*(CB**2+SB**2)*v*C1x(mv9(i9)**2,MVW**2)*
     &  CRFvFemHp(i9,init,1)*CONJG(UV(i9,fin)))/(16d0*Sqrt2*Pi**2*
     &  SW**2)+(e*(3*C11x(mv9(i9)**2,MVW**2)+2*C1x(mv9(i9)**2,MVW**2))*
     &  CLFepFvHm(fin,i9,1)*CRFvFemHp(i9,init,1)*MassFe(fin))/(16d0*Pi*
     &  *2)-(e*(C0x(mv9(i9)**2,MVW**2)+2*C1x(mv9(i9)**2,MVW**2))*
     &  CRFepFvHm(fin,i9,1)*CRFvFemHp(i9,init,1)*mv9(i9))/(8d0*Pi**2)+
     &  (e*MassFe(init)*(C1x(mv9(i9)**2,MVW**2)*(2*SW**2*
     &  CLFvFemHp(i9,init,1)*CRFepFvHm(fin,i9,1)-e**2*CONJG(UV(i9,fin)
     &  )*UV(i9,init))+3*C11x(mv9(i9)**2,MVW**2)*(SW**2*
     &  CLFvFemHp(i9,init,1)*CRFepFvHm(fin,i9,1)+e**2*CONJG(UV(i9,fin)
     &  )*UV(i9,init))))/(16d0*Pi**2*SW**2)

      enddo
          print*, "SMFF(1:2)",SMFF(1:2)
#endif

      SMFF(:)=0D0
      do i9 = 1,9  
       SMFF(1)=SMFF(1)+MassFe(fin)*conjg(UV(i9,fin))*UV(i9,init)*(MVW**2*
     &  (6*C11x(mv9(i9)**2,MVW**2)-4*C1x(mv9(i9)**2,MVW**2))+
     &  (3*C11x(mv9(i9)**2,MVW**2)+2*C1x(mv9(i9)**2,MVW**2))*
     &  MassFe(init)**2+(2*C0x(mv9(i9)**2,MVW**2)+3*C11x(mv9(i9)**
     &  2,MVW**2)+6*C1x(mv9(i9)**2,MVW**2))*mv9(i9)**2)

       SMFF(2)=SMFF(2)+MassFe(init)*conjg(UV(i9,fin))*UV(i9,init)*
     &  ((-2*MVW**2*(-3*C11x(mv9(i9)**2,MVW**2)+(2)*
     &  C1x(mv9(i9)**2,MVW**2)))+(3*C11x(mv9(i9)**2,MVW**2)+2*
     &  C1x(mv9(i9)**2,MVW**2))*MassFe(fin)**2+(2*C0x(mv9(i9)**2,MVW**
     &  2)+3*C11x(mv9(i9)**2,MVW**2)+6*C1x(mv9(i9)**2,MVW**2))*mv9(i9)*
     &  *2)
      enddo
       SMFF(1:2)=e**3/(32d0*pi**2*MVW**2*SW**2)*SMFF(1:2)
!       print*, "SMFF(1:2)",SMFF(1:2)


       HpmFF(:)=0D0
      do i9 = 1,9   
       HpmFF(1)=HpmFF(1)+(e*(3*C11x(mv9(i9)**2,MassHpm(2)**2)+2*C1x(mv9(i9)**2,MassHpm(2)*
     &  *2))*(CLFvFemHp(i9,init,2)*CRFepFvHm(fin,i9,2)*MassFe(fin)+
     &  CLFepFvHm(fin,i9,2)*CRFvFemHp(i9,init,2)*MassFe(init))-2*e*
     &  (C0x(mv9(i9)**2,MassHpm(2)**2)+2*C1x(mv9(i9)**2,MassHpm(2)**2))*
     &  CLFepFvHm(fin,i9,2)*CLFvFemHp(i9,init,2)*mv9(i9))/(16d0*Pi**2)
       HpmFF(2)=HpmFF(2)+(e*(3*C11x(mv9(i9)**2,MassHpm(2)**2)+2*C1x(mv9(i9)**2,MassHpm(2)*
     &  *2))*(CLFepFvHm(fin,i9,2)*CRFvFemHp(i9,init,2)*MassFe(fin)+
     &  CLFvFemHp(i9,init,2)*CRFepFvHm(fin,i9,2)*MassFe(init))-2*e*
     &  (C0x(mv9(i9)**2,MassHpm(2)**2)+2*C1x(mv9(i9)**2,MassHpm(2)**2))*
     &  CRFepFvHm(fin,i9,2)*CRFvFemHp(i9,init,2)*mv9(i9))/(16d0*Pi**2)
      enddo

!      print*, "HpmFF(1:2)",HpmFF(1:2)
      ChaFF(1:2)=0d0
      do i18 = 1,18
      do i2 = 1,2
       ChaFF(1) = ChaFF(1)-(e*(3*C11x(MSv18(i18)**2,MassCha(i2)**2)*(CLF12FemSv(i2,init,i18)
     -       *CRFepF12Sv(fin,i2,i18)*MassFe(fin) + 
     -       CLFepF12Sv(fin,i2,i18)*CRF12FemSv(i2,init,i18)*MassFe(init)) + 
     -       2*C1x(MSv18(i18)**2,MassCha(i2)**2)*(CLF12FemSv(i2,init,i18)
     -       *(2*CLFepF12Sv(fin,i2,i18)*MassCha(i2) + CRFepF12Sv(fin,i2,i18)*MassFe(fin)) + 
     -       CLFepF12Sv(fin,i2,i18)*CRF12FemSv(i2,init,i18)*MassFe(init))))/(16.*Pi**2)
       ChaFF(2) = ChaFF(2)-(2*e*(CRF12FemSv(i2,init,i18)*(2*CRFepF12Sv(fin,i2,i18)*
     -        MassCha(i2) + CLFepF12Sv(fin,i2,i18)*MassFe(fin)) + 
     -        CLF12FemSv(i2,init,i18)*CRFepF12Sv(fin,i2,i18)*MassFe(init))
     -       *C1x(MSv18(i18)**2,MassCha(i2)**2) + 
     -       3*e*(CLFepF12Sv(fin,i2,i18)*CRF12FemSv(i2,init,i18)*MassFe(fin) 
     -       + CLF12FemSv(i2,init,i18)*CRFepF12Sv(fin,i2,i18)*MassFe(init))*
     -       C11x(MSv18(i18)**2,MassCha(i2)**2))/(16.*Pi**2)
      enddo
      enddo

!      print*, "ChaFF(1),ChaFF(2)",ChaFF(1:2)

c --- Finalize
      FF(:) = ( SMFF(:) + HpmFF(:) + ChaFF(:))/2.0


      if(init.eq.2 .and. fin.eq.1)then
          brin= 1.0
      elseif (init.eq.3 .and. fin.eq.2)then   
          brin= 0.1782d0
      elseif (init.eq.3 .and. fin.eq.1)then   
          brin= 0.1739d0
      endif

      output= (12.0*pi**2)*(CONJG(FF(1))*FF(1) + CONJG(FF(2))*FF(2))
     -         /(GF**2*MassFe(init)**2)*brin 
c --- Check positive decay rate ---
      if (imag(output) .lt. 1d-9 .and. real(output) .ge. 0d0) then
        Lep2LepGamDecay = real(output)
      else
        print*, '* WARNING! Non-positive decay rate. Obtained ', output
        Lep2LepGamDecay = 0d0
      endif
      
      return

      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      subroutine Yvoutput(Yvout,Yvmax,iimax,jjmax)
      implicit none
      integer i,j,iimax,jjmax
      double precision Yvmax
      double complex Yvout(3,3)

#include "issnmssm.h"
      yvmax=abs(Yv(1,1))
      iimax=1;jjmax=1
      do i = 1,3
        do j = 1,3
		  if(abs(Yv(i,j)) .gt. Yvmax)then
	        Yvmax=abs(Yv(i,j))
            iimax=i;jjmax=j
          endif
        enddo
      enddo
	  Yvout=Yv

      end
      
      
