* This file is a part of NMSSMCALC-nuSS 
* It contains:
* subroutine dmtau_EWcnmssm: compute dmtau in NMSSM-ISS
* subroutine Delta_CHlnu: compute decay width of H+->l +nu in NMSSM-ISS
* subroutine Delta_CHstausneu: compute decay width of H+->l~ +nu~ in NMSSM-ISS
* subroutine Delta_Hsneusneu: compute decay width of H->nu~ +nu~ in NMSSM-ISS
* subroutine Delta_Hnunu: compute decay width of H->nu +nu in NMSSM-ISS
c last modified by ND 7/2021
c*********************************************************************c
#include "def.h"
      subroutine dmtau_EWcnmssm(deltab,tanb,il)
      implicit none
      integer il
      double precision tanb,alpha1,alpha2,am1abs,am2abs,amu,cb,cw
     .      ,pi,sb,sneut,stau1,stau2,sw,v,vs,xphim1,xphim2
      double precision t_hdec
      double precision gf,alph,amtau,ammuon,amz,amw
      double precision xlambdapar,xkappa,Alam2,Akap2,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      double precision xlambdaparim,xkappaim,Alamim,Akapim,
     .     xlamvsim,xifim,xisim,xmuprimeim,amsph2im,amsh2im,xphiu
      double precision am1realval,am1imval,am2realval,am2imval
      double precision amchar(2),amneut(5),xmneut(5),
     .          xmst(2),xmsb(2),amsl(2),
     .          amsu(2),amsd(2),amse(2),amsn(18),amsnl(2),amnu(9)
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      double complex myi,deltab

      common/param_hdec/gf,alph,amtau,ammuon,amz,amw
      common/extpar_nmssm/xlambdapar,xkappa,Alam2,Akap2,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      common/extpart_cnmssm_im/xlambdaparim,xkappaim,Alamim,Akapim,
     .     xlamvsim,xifim,xisim,xmuprimeim,amsph2im,amsh2im,xphiu
      common/smass_nmssm/amneut,xmneut,amchar,xmst,xmsb,amsl,
     .              amsu,amsd,amse,amsn,amsnl,amnu
      COMMON/M1M2VALUES_CNMSSM/am1realval,am1imval,am2realval,am2imval
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      myi = dcmplx(0.d0,1.d0)
      pi = 4*datan(1.d0)
      v = 1/dsqrt(gf*dsqrt(2.d0))
      vs = dsqrt(2.d0*(xlamvs**2+xlamvsim**2)/
     .     (xlambdapar**2+xlambdaparim**2))
      sb = tanb/dsqrt(1+tanb**2)
      cb = 1.d0/dsqrt(1+tanb**2)

      amu = dsqrt(xlamvs**2+xlamvsim**2)
      if(il.eq.0) then
         stau1 = amse(1)
         stau2 = amse(2)
         sneut = dsqrt(ml2(1))
      else
         stau1 = amsl(1)
         stau2 = amsl(2)
         sneut = dsqrt(ml2(3))
      endif

      cw = amw**2/amz**2
      sw = 1-cw
      alpha2 = (2*amw/v)**2/4/pi

      alpha1 = alpha2*sw/cw
      am2abs = dsqrt(am2realval**2+am2imval**2) 
      am1abs = dsqrt(am1realval**2+am1imval**2) 

      xphim2 = datan(am2imval/am2realval)
      xphim1 = datan(am1imval/am1realval)

      deltab = alpha1/4/pi*am1abs*(
     .     xlamvs*dcos(xphim1)-xlamvsim*dsin(xphim1)-
     .     myi*(xlamvsim*dcos(xphim1)+xlamvs*dsin(xphim1)))*
     .     tanb*t_hdec(stau1,stau2,am1abs)
     .     - alpha2/4/pi*am2abs*(
     .     xlamvs*dcos(xphim2)-xlamvsim*dsin(xphim2)-
     .     myi*(xlamvsim*dcos(xphim2)+xlamvs*dsin(xphim2)))*
     .     tanb*t_hdec(sneut,am2abs,amu)

      return
      end
c---------------------------------------------------------------------c

      subroutine dmtau_EWnmssm(deltab,tanb,il)
      implicit none
      integer il
      double precision tanb,alpha1,alpha2,am1abs,am2abs,amu,cw
     .      ,pi,sneut,stau1,stau2,sw,v,xphim1,xphim2
      double precision t_hdec
      double precision gf,alph,amtau,ammuon,amz,amw
      double precision xlambdapar,xkappa,Alam2,Akap2,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      double precision am1realval,am1imval,am2realval,am2imval
      double precision amchar(2),amneut(5),xmneut(5),
     .          xmst(2),xmsb(2),amsl(2),
     .          amsu(2),amsd(2),amse(2),amsn(18),amsnl(2),amnu(9)
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      double complex deltab

      common/param_hdec/gf,alph,amtau,ammuon,amz,amw
      common/extpar_nmssm/xlambdapar,xkappa,Alam2,Akap2,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      common/smass_nmssm/amneut,xmneut,amchar,xmst,xmsb,amsl,
     .              amsu,amsd,amse,amsn,amsnl,amnu
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2

      pi = 4*datan(1.d0)
      v = 1/dsqrt(gf*dsqrt(2.d0))

      amu = dsqrt(xlamvs**2)
      if(il.eq.0) then
         stau1 = amse(1)
         stau2 = amse(2)
         sneut = dsqrt(ml2(1))
      else
         stau1 = amsl(1)
         stau2 = amsl(2)
         sneut = dsqrt(ml2(3))
      endif

      cw = amw**2/amz**2
      sw = 1-cw
      alpha2 = (2*amw/v)**2/4/pi
      alpha1 = alpha2*sw/cw
      am2abs = absM1
      am1abs = absM2 

      xphim2 = datan(am2imval/am2realval)
      xphim1 = datan(am1imval/am1realval)

      deltab = alpha1/4/pi*am1abs*amu*
     .     tanb*t_hdec(stau1,stau2,am1abs)
     .     - alpha2/4/pi*am2abs*amu*
     .     tanb*t_hdec(sneut,am2abs,amu)

      return
      end

c---------------------------------------------------------------------c
      subroutine Delta_CHlnu(awidth,deltab,ilevel,igen,inue)
        implicit none 
        integer igen,ilevel,inue
        double precision MF,MFP,MCH2,MF2,MFP2,treepart,RPhaseSpace2,
     &                   awidth,CTree
        double complex TreeL,TreeR,deltab
        double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
        double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
        double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
        common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
        common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
        common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta

#include "issnmssm.h"
        if(igen.eq.2)then
          MF=mmu
        elseif (igen.eq.3)then
          MF=mtau
        endif
        MFp=mv9(inue)
        MF2=MF**2
        MFp2=MFp**2
        MCH2=MHpm**2
        RPhaseSpace2=sqrt(MCH2**2+MF**4+MFP**4
     &   -2D0*MCH2*MF**2-2D0*MCH2*MFP**2-2D0*MF**2*MFP**2)/16D0/pi/MHPm**3
        TreeL=CLFvFemHp(inue,igen,2)
        TreeR=CRFvFemHp(inue,igen,2)*(1-deltab/tan(beta)**2)/(1+deltab)
        Ctree=DBLE(-2D0*MF*MFP*TreeL*DCONJG(TreeR)
     &   -2D0*MF*MFP*TreeR*DCONJG(TreeL)+
     &   (MCH2-MF**2-MFP**2)*(TreeL*DCONJG(TreeL)+TreeR*DCONJG(TreeR)))
        treepart=RPhaseSpace2*Ctree
        awidth=treepart
      end subroutine Delta_CHlnu
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine Delta_CHstausneu(MHp,MX1,MX2,iin,jin,awidth,ischeme,ilevel)
        implicit none 
        integer ischeme,ilevel,iin,jin
        double precision MHp,MX1,MX2,awidth,treepart,RPhasespace2,MX12,
     .                   MX22,MCH2
        double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     .                 MSu(6),MSv(3)
        double complex tree
        common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
#include "issnmssm.h"
        MX22=MX2**2;MX12=MX1**2
        MCH2=MHpm**2
        tree=-zi*CSemSvHp(iin,jin,2)
        RPhasespace2=1D0/16D0/Pi/MHpm**3*Sqrt(MCH2**2+MX12**2+MX22**2-2*MCH2*MX12
     &    -2*MCH2*MX22-2*MX12*MX22)
        treepart = RPhasespace2*abs(Tree)**2
        awidth=treepart
        end subroutine Delta_CHstausneu

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine Delta_Hsneusneu(MHin,MX1,MX2,i1,j1,j2,awidth,ischeme,ilevel)
        implicit none 
        integer ischeme,ilevel,i1,j1,j2,i,imin,ima
        double precision MHin,MX1,MX2,awidth,treepart,RPhasespace2,MX12,
     .                   MX22,MH2
        double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     .                 MSu(6),MSv(3)
        double complex tree,ZHfac(5,5)

        logical CPcon,MHpmInput,onelswitch,mQrun
        common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
        common /Zfactor/ ZHfac

        common /switch/ MHpmInput,CPcon,mQrun,onelswitch
#include "issnmssm.h"
        MX22=MX2**2;MX12=MX1**2
        MH2=MHin**2
        if(CPcon)then
         if (i1.lt.3)then
          imin=1;ima=2
         else
          imin=3;ima=5
         endif
        else
        imin=1;ima=5
        endif
        tree=0D0
        do i=imin,ima
        tree=tree+ ZHfac(i1,i)*(-zi*ChSvSv(i+1,j1,j2))
        enddo

        RPhasespace2=1D0/16D0/Pi/MHin**3*Sqrt(MH2**2+MX12**2+MX22**2-2*MH2*MX12
     &    -2*MH2*MX22-2*MX12*MX22)
        treepart = RPhasespace2*abs(Tree)**2
        awidth=treepart
        end subroutine Delta_Hsneusneu

c---------------------------------------------------------------------c
c---------------------------------------------------------------------c
      subroutine Delta_Hnunu(MHin,iin,inu1,inu2,awidth,ilevel)
        implicit none 
        integer iin,ilevel,inu1,inu2,i,imin,ima
        double precision MF,MFP,MHin,MH2,MF2,MFP2,treepart,RPhaseSpace2,
     &                   awidth,CTree
        double complex TreeL,TreeR,ZHfac(5,5)
        double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
        double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
        double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
        logical CPcon,MHpmInput,onelswitch,mQrun

        common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
        common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
        common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
        common /Zfactor/ ZHfac

        common /switch/ MHpmInput,CPcon,mQrun,onelswitch
#include "issnmssm.h"
        MF=mv9(inu1)
        MFp=mv9(inu2)
        MF2=MF**2
        MFp2=MFp**2
        MH2=MHin**2
        RPhaseSpace2=sqrt(MH2**2+MF**4+MFP**4
     &   -2D0*MH2*MF**2-2D0*MH2*MFP**2-2D0*MF**2*MFP**2)/16D0/pi/MHin**3

        if(CPcon)then
         if (iin.lt.3)then
          imin=1;ima=2
         else
          imin=3;ima=5
         endif
        else
        imin=1;ima=5
        endif
        TreeL=0D0
        TreeR=0D0
        do i=imin,ima
        TreeL=TreeL+ZHfac(iin,i)*CLFvFvh(inu1,inu2,i+1)
        TreeR=TreeR+ZHfac(iin,i)*CRFvFvh(inu1,inu2,i+1)
        enddo

        Ctree=DBLE(-2D0*MF*MFP*TreeL*DCONJG(TreeR)
     &   -2D0*MF*MFP*TreeR*DCONJG(TreeL)+
     &   (MH2-MF**2-MFP**2)*(TreeL*DCONJG(TreeL)+TreeR*DCONJG(TreeR)))
             
        treepart=RPhaseSpace2*Ctree
        awidth=treepart
      end subroutine Delta_Hnunu



