* This file is a part of NMSSMCALC-nuSS 
* It contains:
* subroutine ismSTU_check: compute STU parameters of the SM
* double precision function ismDZself: compute derivative of Z-selfenergy of the SM
* double precision function ismYself: compute gamma-selfenergy  of the SM
* double precision function ismDYself: compute derivative of Z-selfenergy  of the SM
* double precision function ismZself: compute Z-selfenergy  of the SM
* double precision function ismWself: compute W-selfenergy  of the SM
* double precision function ismDWself: compute derivative of W-selfenergy  of the SM
* subroutine ismSTU_UVcheck: check UV finite of STU parameters of the SM
c last modified by ND 7/2021
c*********************************************************************c
#include "def.h"
      subroutine ismSTU_check()
      implicit none
      integer i,j,gt1,gt2,gt3,gt4

      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
           

      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3

      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu

      double precision ismZself,ismyself,ismWself,ismyzself,
     &                ismdzself,ismdyself,ismdwself,ismdyzself,
     &                S,T,U
      double precision selfz,selfy,selfw,selfyz,
     &                 dselfz,dselfy,dselfw,dselfyz,selfQQ,self3Q,
     &                 self33,self11,dselfQQ,dself3Q,
     &                 dself33,dself11 
#include "issnmssm.h"
#include "HdecayTCLs.inc"

       alfa= e**2/4D0/pi
       MH2=125D0**2
c       print*, "T", (selfw/MW2-selfz/MZ2 ) /alfa,- mt**2*3/16/pi**2/v**2 /alfa
       selfz=ismZself(0D0)
       selfw=ismwself(0D0)
       selfy=ismyself(0D0)
       selfyz=ismyZself(0D0)

       dselfz=(ismZself(MZ2)-ismZself(0D0))/MZ2
       dselfw=(ismwself(MW2)-ismwself(0D0))/MW2
       dselfy=ismdyself(0D0)
       dselfyz=ismdyZself(0D0)

c       selfQQ= selfy/e**2
c       self3Q=(cw*selfyz*sw)/e**2 + (selfy*sw**2)/e**2
       self11=(selfw*sw**2)/e**2
       self33=(cw**2*selfz*sw**2)/e**2 + 
     -    (2*cw*selfyz*sw**3)/e**2 + (selfy*sw**4)/e**2

c       dselfQQ= dselfy/e**2
       dself3Q=(cw*dselfyz*sw)/e**2 + (dselfy*sw**2)/e**2
       dself11=(dselfw*sw**2)/e**2
       dself33=(cw**2*dselfz*sw**2)/e**2 + 
     -    (2*cw*dselfyz*sw**3)/e**2 + (dselfy*sw**4)/e**2


       S= 16*pi*(dself33 -dself3Q)
       T= 4*pi/sw**2/cw**2/MZ2*(self11 -self33)
       U = 16*pi*(dself11 -dself33)
#ifdef detalis
       print*, "=======================SM==========================="
       print*, "Follow definitions in  M. E. Peskin and T. Takeuchi 1992, "
       print*, "T", (selfw/MW2-selfz/MZ2 ) /alfa,- mt**2*3/16/pi**2/v**2 /alfa

       print*, "S",4*sw**2*cw**2*4*pi/e**2 *(dselfz - (cw**2-sw**2)/cw/sw*dselfyz -dselfy)
       print*, "U", 4*sw**2*4*pi/e**2*(dselfw - cw**2*dselfz - sw**2*dselfy- 2*cw*sw*dselfyz)

       print*, "selfz,selfw,selfy,selfyz",selfz,selfw,selfy,selfyz
       print*, "dselfz,dselfw,dselfy,dselfyz",dselfz,dselfw,dselfy,dselfyz
       print*, "S,T,U= ",S,T,U
#endif
      end



c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function ismDZself(p2)
        implicit none
        integer i,j,gt1,gt2,gt3,gt4
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      Dself=0D0
       Dself=Dself+(-4*Alfa*Pi*(2*CW2*Finite+6*CW2*
     &  myB0i(bb1,P2,MW2,MW2)-(3*MW2*myB0i(dbb0,P2,MH2,MZ2))/CW2**2+3*
     &  (9*CW2-2*SW2+SW2**2/CW2)*myB0i(dbb00,P2,MW2,MW2)+(3*(10*CW2**2*
     &  myB0i(bb0,P2,MW2,MW2)+3*myB0i(bb1,P2,0D0,0D0)+4*CW2**2*MW2*
     &  myB0i(dbb0,P2,MW2,MW2)+10*CW2**2*P2*myB0i(dbb0,P2,MW2,MW2)-4*
     &  MW2*SW2**2*myB0i(dbb0,P2,MW2,MW2)-6*myB0i(dbb00,P2,0D0,0D0)+2*
     &  myB0i(dbb00,P2,MH2,MZ2)+3*P2*myB0i(dbb1,P2,0D0,0D0)))/(2d0*CW2)+
     &  6*CW2*P2*myB0i(dbb1,P2,MW2,MW2)))/(3d0*SW2)
           do Gen3 = 1,3
       Dself=Dself+(-2*Alfa*Pi*(((9-12*SW2+8*SW2**2)*
     &  myB0i(bb1,P2,MFd2(Gen3),MFd2(Gen3)))/SW2-3*(4-1/SW2-8*SW2)*
     &  myB0i(bb1,P2,MFe2(Gen3),MFe2(Gen3))+((9-24*SW2+32*SW2**2)*
     &  myB0i(bb1,P2,MFu2(Gen3),MFu2(Gen3)))/SW2+(9*MFd2(Gen3)*
     &  myB0i(dbb0,P2,MFd2(Gen3),MFd2(Gen3))+3*MFe2(Gen3)*
     &  myB0i(dbb0,P2,MFe2(Gen3),MFe2(Gen3))+9*MFu2(Gen3)*
     &  myB0i(dbb0,P2,MFu2(Gen3),MFu2(Gen3)))/SW2+6*(4-1/SW2-8*SW2)*
     &  myB0i(dbb00,P2,MFe2(Gen3),MFe2(Gen3))+2*((12-9/SW2-8*SW2)*
     &  myB0i(dbb00,P2,MFd2(Gen3),MFd2(Gen3))+(24-9/SW2-32*SW2)*
     &  myB0i(dbb00,P2,MFu2(Gen3),MFu2(Gen3)))+(P2*((9-12*SW2+8*SW2**2)
     &  *myB0i(dbb1,P2,MFd2(Gen3),MFd2(Gen3))+3*(1-4*SW2+8*SW2**2)*
     &  myB0i(dbb1,P2,MFe2(Gen3),MFe2(Gen3))+(9-24*SW2+32*SW2**2)*
     &  myB0i(dbb1,P2,MFu2(Gen3),MFu2(Gen3))))/SW2))/(3d0*CW2)
           enddo
      ismDZself=Dself*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function ismYself(p2)
        implicit none
        integer i,j,gt1,gt2,gt3,gt4
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      self=0D0
       self=self+(-4*Alfa*Pi*(2*Finite*P2-18*myA0(MW2)+15*P2*
     &  myB0i(bb0,P2,MW2,MW2)+36*myB0i(bb00,P2,MW2,MW2)+6*P2*
     &  myB0i(bb1,P2,MW2,MW2)))/3d0
           do Gen3 = 1,3
       self=self+(-16*Alfa*Pi*(myA0(MFd2(Gen3))+3*myA0(MFe2(Gen3))+4*
     &  myA0(MFu2(Gen3))-2*myB0i(bb00,P2,MFd2(Gen3),MFd2(Gen3))-6*
     &  myB0i(bb00,P2,MFe2(Gen3),MFe2(Gen3))-8*myB0i(bb00,P2,MFu2(Gen3)
     &  ,MFu2(Gen3))+P2*myB0i(bb1,P2,MFd2(Gen3),MFd2(Gen3))+3*P2*
     &  myB0i(bb1,P2,MFe2(Gen3),MFe2(Gen3))+4*P2*
     &  myB0i(bb1,P2,MFu2(Gen3),MFu2(Gen3))))/3d0
           enddo
      ismYself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function ismDYself(p2)
        implicit none
        integer i,j,gt1,gt2,gt3,gt4
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      Dself=0D0
       Dself=Dself+(-4*Alfa*Pi*(2*Finite+15*myB0i(bb0,P2,MW2,MW2)+6*
     &  myB0i(bb1,P2,MW2,MW2)+15*P2*myB0i(dbb0,P2,MW2,MW2)+36*
     &  myB0i(dbb00,P2,MW2,MW2)+6*P2*myB0i(dbb1,P2,MW2,MW2)))/3d0
           do Gen3 = 1,3
       Dself=Dself+(-16*Alfa*Pi*(myB0i(bb1,P2,MFd2(Gen3),MFd2(Gen3))+3*
     &  myB0i(bb1,P2,MFe2(Gen3),MFe2(Gen3))+4*myB0i(bb1,P2,MFu2(Gen3)
     &  ,MFu2(Gen3))-2*myB0i(dbb00,P2,MFd2(Gen3),MFd2(Gen3))-6*
     &  myB0i(dbb00,P2,MFe2(Gen3),MFe2(Gen3))-8*
     &  myB0i(dbb00,P2,MFu2(Gen3),MFu2(Gen3))+P2*
     &  myB0i(dbb1,P2,MFd2(Gen3),MFd2(Gen3))+3*P2*
     &  myB0i(dbb1,P2,MFe2(Gen3),MFe2(Gen3))+4*P2*
     &  myB0i(dbb1,P2,MFu2(Gen3),MFu2(Gen3))))/3d0
           enddo
      ismDYself=Dself*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function ismYZself(p2)
      implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision ZH5(5,5),ZH(6,6)
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      double precision p2,self
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      self=0D0
       self=self+(4*Alfa*Pi*(2*CW2*Finite*P2+(3-18*CW2)*myA0(MW2)+3*(2*
     &  CW2*MW2+5*CW2*P2+2*MW2*SW2)*myB0i(bb0,P2,MW2,MW2)+30*CW2*
     &  myB0i(bb00,P2,MW2,MW2)-6*SW2*myB0i(bb00,P2,MW2,MW2)+6*CW2*P2*
     &  myB0i(bb1,P2,MW2,MW2)))/(3d0*CW*SW)

           do Gen3 = 1,3
       self=self+(-4*Alfa*Pi*((-3+4*SW2)*(myA0(MFd2(Gen3))-2*
     &  myB0i(bb00,P2,MFd2(Gen3),MFd2(Gen3)))+3*(-1+4*SW2)*
     &  (myA0(MFe2(Gen3))-2*myB0i(bb00,P2,MFe2(Gen3),MFe2(Gen3)))+2*(-
     &  3+8*SW2)*(myA0(MFu2(Gen3))-2*myB0i(bb00,P2,MFu2(Gen3)
     &  ,MFu2(Gen3)))+P2*((-3+4*SW2)*myB0i(bb1,P2,MFd2(Gen3),MFd2(Gen3)
     &  )+3*(-1+4*SW2)*myB0i(bb1,P2,MFe2(Gen3),MFe2(Gen3))+2*(-3+8*SW2)
     &  *myB0i(bb1,P2,MFu2(Gen3),MFu2(Gen3)))))/(3d0*CW*SW)
           enddo
      ismYZself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function ismDYZself(p2)
      implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      Dself=0D0
       Dself=Dself+(4*Alfa*Pi*(2*CW2*Finite+15*CW2*
     &  myB0i(bb0,P2,MW2,MW2)+6*CW2*myB0i(bb1,P2,MW2,MW2)+6*CW2*MW2*
     &  myB0i(dbb0,P2,MW2,MW2)+15*CW2*P2*myB0i(dbb0,P2,MW2,MW2)+6*MW2*
     &  SW2*myB0i(dbb0,P2,MW2,MW2)+30*CW2*myB0i(dbb00,P2,MW2,MW2)-6*
     &  SW2*myB0i(dbb00,P2,MW2,MW2)+6*CW2*P2*myB0i(dbb1,P2,MW2,MW2)))
     &  /(3d0*CW*SW)
           do Gen3 = 1,3
       Dself=Dself+(-4*Alfa*Pi*((-3+4*SW2)*myB0i(bb1,P2,MFd2(Gen3)
     &  ,MFd2(Gen3))+3*(-1+4*SW2)*myB0i(bb1,P2,MFe2(Gen3),MFe2(Gen3))+
     &  2*(-3+8*SW2)*myB0i(bb1,P2,MFu2(Gen3),MFu2(Gen3))+2*(3-4*SW2)*
     &  myB0i(dbb00,P2,MFd2(Gen3),MFd2(Gen3))+6*(1-4*SW2)*
     &  myB0i(dbb00,P2,MFe2(Gen3),MFe2(Gen3))+4*(3-8*SW2)*
     &  myB0i(dbb00,P2,MFu2(Gen3),MFu2(Gen3))+P2*((-3+4*SW2)*
     &  myB0i(dbb1,P2,MFd2(Gen3),MFd2(Gen3))+3*(-1+4*SW2)*
     &  myB0i(dbb1,P2,MFe2(Gen3),MFe2(Gen3))+2*(-3+8*SW2)*
     &  myB0i(dbb1,P2,MFu2(Gen3),MFu2(Gen3)))))/(3d0*CW*SW)
           enddo
      ismDYZself=Dself*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        double precision function ismZself(p2)
        implicit none
        integer i,j,gt1,gt2,gt3,gt4
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,self,faca0,facb0,facb1,facb00
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      self=0D0
       faca0=1
       facb0=1
       facb1=1
       facb00=1
       self=self-(Alfa*Pi*(8*CW2*Finite*P2+6*(3-12*CW2-SW2/CW2)*
     &  faca0*myA0(MW2)-(3*(faca0*myA0(MH2)+faca0*myA0(MZ2)))/CW2-(12*MW2*
     &  facb0*myB0i(bb0,P2,MH2,MZ2))/CW2**2+12*(9*CW2-2*SW2+SW2**2/CW2)*
     &  facb00*myB0i(bb00,P2,MW2,MW2)+(6*(2*(CW2**2*(2*MW2+5*P2)-2*MW2*SW2**2)
     &  *facb0*myB0i(bb0,P2,MW2,MW2)-6*facb00*myB0i(bb00,P2,0D0,0D0)+2*
     &  facb00*myB0i(bb00,P2,MH2,MZ2)+3*P2*facb1*myB0i(bb1,P2,0D0,0D0)))/CW2+24*CW2*
     &  P2*facb1*myB0i(bb1,P2,MW2,MW2)))/(3d0*SW2)

c        print*, "selfZ", self*fac/MZ2 /alfa
 
c         print*, "denner",-alfa/4D0/pi* (1/6D0/sw2/cw2*( ( 24*cw2**2+16*cw2-10)*mw2*
c     &           myB0i(bb0,p2,MW2,MW2)-(24*cw2**2-8*cw2+2)*mw2*myB0i(bb0,0D0,MW2,MW2)
c     &         )+
c     &         1/12D0/sw2/cw2*((2*MH2-10*MZ2-p2)*myB0i(bb0,p2,mz2,mh2) 
c     &         -2*mz2*myB0i(bb0,0D0,mz2,mz2) -2*Mh2*myB0i(bb0,0D0,mh2,mh2)
c     &         - 2/3D0*p2 - (mz2-mh2)**2*myb0i(dbb0,0D0,mz2,mh2) ))/MZ2 /alfa
           do Gen3 = 1,3
       self=self+(-2*Alfa*Pi*(((9-12*SW2+8*SW2**2)*myA0(MFd2(Gen3))+(9-
     &  24*SW2+32*SW2**2)*myA0(MFu2(Gen3)))/SW2+(9*MFd2(Gen3)*
     &  myB0i(bb0,P2,MFd2(Gen3),MFd2(Gen3))+3*MFe2(Gen3)*
     &  myB0i(bb0,P2,MFe2(Gen3),MFe2(Gen3))+9*MFu2(Gen3)*
     &  myB0i(bb0,P2,MFu2(Gen3),MFu2(Gen3)))/SW2+(3*(1-4*SW2+8*SW2**2)*
     &  (myA0(MFe2(Gen3))-2*myB0i(bb00,P2,MFe2(Gen3),MFe2(Gen3))))/SW2+
     &  2*((12-9/SW2-8*SW2)*myB0i(bb00,P2,MFd2(Gen3),MFd2(Gen3))+(24-
     &  9/SW2-32*SW2)*myB0i(bb00,P2,MFu2(Gen3),MFu2(Gen3)))+(P2*((9-12*
     &  SW2+8*SW2**2)*myB0i(bb1,P2,MFd2(Gen3),MFd2(Gen3))+3*(1-4*SW2+8*
     &  SW2**2)*myB0i(bb1,P2,MFe2(Gen3),MFe2(Gen3))+(9-24*SW2+32*SW2**
     &  2)*myB0i(bb1,P2,MFu2(Gen3),MFu2(Gen3))))/SW2))/(3d0*CW2)
           enddo
       ismZself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function ismWself(p2)
      implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,self,faca0,facb0,facb1,facb00
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
       self=0D0
       faca0=1
       facb0=1
       facb1=1
       facb00=1
       self=self-4*Alfa*Pi*(-((MW2-5*P2)*facb0*myB0i(bb0,P2,0D0,MW2))+8*
     &  facb00*myB0i(bb00,P2,0D0,MW2)+(facb00*myB0i(bb00,P2,MH2,MW2)+(1+8*CW2)*
     & facb00*myB0i(bb00,P2,MW2,MZ2))/SW2+2*P2*facb1*myB0i(bb1,P2,0D0,MW2)-(-8*
     &  Finite*P2+18*faca0*myA0(MW2)+3*(faca0*myA0(MH2)+faca0*myA0(MZ2)+12*CW2*faca0*myA0(MZ2))
     &  +12*MW2*facb0*myB0i(bb0,P2,MH2,MW2)+3*(-4*CW2*(2*MZ2+3*P2)+(4*MW2*
     &  SW2**2)/CW2)*facb0*myB0i(bb0,P2,MW2,MZ2)+24*CW2*P2*
     &  facb1*myB0i(bb1,P2,MW2,MZ2))/(12d0*SW2))
c         print*, "selfW", self*fac/MW2 /alfa


c       print*, -(Alfa*Pi*(8*CW2**2*P2+8*CW2*P2*SW2-3*CW2*myA0(MH2)+6*
c     &  CW2*(-7+4*CW2+4*SW2)*myA0(MW2)-3*CW2*myA0(MZ2)-36*CW2**2*
c     &  myA0(MZ2)-12*CW2*MW2*SW2*myB0i(bb0,P2,0D0,MW2)+60*CW2*P2*SW2*
c     &  myB0i(bb0,P2,0D0,MW2)-12*CW2*MW2*myB0i(bb0,P2,MH2,MW2)+24*CW2**
c     &  2*MZ2*myB0i(bb0,P2,MW2,MZ2)+36*CW2**2*P2*myB0i(bb0,P2,MW2,MZ2)-
c     &  12*MW2*SW2**2*myB0i(bb0,P2,MW2,MZ2)+96*CW2*SW2*
c     &  myB0i(bb00,P2,0D0,MW2)+12*CW2*myB0i(bb00,P2,MH2,MW2)+12*CW2*
c     &  myB0i(bb00,P2,MW2,MZ2)+96*CW2**2*myB0i(bb00,P2,MW2,MZ2)+24*CW2*
c     &  P2*SW2*myB0i(bb1,P2,0D0,MW2)-24*CW2**2*P2*myB0i(bb1,P2,MW2,MZ2)
c     &  ))/(3d0*CW2*SW2)*fac/MW2 /alfa

c       print*, "denner", -alfa/4D0/pi*(2/3d0*(2*MW2*myB0i(bb0,p2,Mw2,0D0)-
c     &           2*mw2*myb0i(bb0,0D0,mw2,mw2) -mw2**2*myb0i(dbb0, p2,mw2,0d0) ) 
c     &          +1/12D0/sw2*((16*cw2+54-10d0/cw2)*MW2*myb0i(bb0,p2,mw2,mz2)
c     &         -(16*cw2+2)*(mw2*myb0i(bb0,0D0,mw2,mw2)+mz2*myb0i(bb0,p2,mz2,mz2))
c     &        -(8*cw2+1)*(mw2-mz2)**2*myb0i(dbb0,p2,mw2,mz2) )
c     &        + 1/12d0/sw2*((2*mh2-10*mw2)*myb0i(bb0,p2,mw2,mh2)
c     &          -2*mw2*myb0i(bb0,0D0,mw2,mw2) -2*mh2*myb0i(bb0,0D0,mh2,mh2)
c     &          - (mw2-mh2)**2*myb0i(dbb0, p2,mw2,mh2) ) )/MW2 /alfa
c         stop
           do Gen3 = 1,3
       self=self+(-4*Alfa*Pi*(3*myA0(MFd2(Gen3))+myA0(MFe2(Gen3))+3*
     &  MFu2(Gen3)*myB0i(bb0,P2,MFd2(Gen3),MFu2(Gen3))-2*
     &  myB0i(bb00,P2,0D0,MFe2(Gen3))-6*myB0i(bb00,P2,MFu2(Gen3)
     &  ,MFd2(Gen3))+P2*myB0i(bb1,P2,0D0,MFe2(Gen3))+3*P2*
     &  myB0i(bb1,P2,MFu2(Gen3),MFd2(Gen3))))/SW2
           enddo
      ismWself=self*fac
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      double precision function ismDWself(p2)
      implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      double precision p2,Dself
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"
      Dself=0D0
       Dself=Dself-4*Alfa*Pi*(5*myB0i(bb0,P2,0D0,MW2)+2*
     &  myB0i(bb1,P2,0D0,MW2)-(MW2-5*P2)*myB0i(dbb0,P2,0D0,MW2)+8*
     &  myB0i(dbb00,P2,0D0,MW2)+(myB0i(dbb00,P2,MH2,MW2)+(1+8*CW2)*
     &  myB0i(dbb00,P2,MW2,MZ2))/SW2+2*P2*myB0i(dbb1,P2,0D0,MW2)-(-8*
     &  Finite-36*CW2*myB0i(bb0,P2,MW2,MZ2)+24*CW2*
     &  myB0i(bb1,P2,MW2,MZ2)+12*MW2*myB0i(dbb0,P2,MH2,MW2)+3*(-4*CW2*
     &  (2*MZ2+3*P2)+(4*MW2*SW2**2)/CW2)*myB0i(dbb0,P2,MW2,MZ2)+24*CW2*
     &  P2*myB0i(dbb1,P2,MW2,MZ2))/(12d0*SW2))
           do Gen3 = 1,3
       Dself=Dself+(-4*Alfa*Pi*(myB0i(bb1,P2,0D0,MFe2(Gen3))+3*
     &  myB0i(bb1,P2,MFu2(Gen3),MFd2(Gen3))+3*MFu2(Gen3)*
     &  myB0i(dbb0,P2,MFd2(Gen3),MFu2(Gen3))-2*
     &  myB0i(dbb00,P2,0D0,MFe2(Gen3))-6*myB0i(dbb00,P2,MFu2(Gen3)
     &  ,MFd2(Gen3))+P2*myB0i(dbb1,P2,0D0,MFe2(Gen3))+3*P2*
     &  myB0i(dbb1,P2,MFu2(Gen3),MFd2(Gen3))))/SW2
           enddo
      ismDWself=Dself*fac
      end



c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine ismSTU_UVcheck()
        implicit none
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision ZH5(5,5),ZH(6,6)
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv

      double precision p2,self,dself,selfz,selfy,selfw,selfyz,
     &                 dselfz,dselfy,dselfw,dselfyz,selfQQ,self3Q,
     &                 self33,self11,dselfQQ,dself3Q,
     &                 dself33,dself11,S,T,U
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
#include "issnmssm.h"
#include "HdecayTCLs.inc"

      
      Dself=0D0
      self=0D0

       Dself=(-10*CWWZ**2)/3d0+Dself
           do I183 = 1,18
           do I184 = 1,18
       Dself=Dself-CSvSvVZ(I183,I184)**2/6d0
           enddo
           enddo
           do I2G3 = 1,2
       Dself=CHpHmZ**2/3d0+Dself
           enddo
           do I2G3 = 1,2
           do I2G4 = 1,2
       Dself=Dself+(2*(CF12ZL(I2G3,I2G4)*CF12ZL(I2G4,I2G3)+
     &  CF12ZR(I2G3,I2G4)*CF12ZR(I2G4,I2G3)))/3d0
           enddo
           enddo
           do I3G3 = 1,3
       Dself=(2*(3*CddZL**2+3*CddZR**2+ctautauZL**2+ctautauZR**2+3*
     &  CuuZL**2+3*CuuZR**2))/3d0+Dself
           enddo
           do I5G3 = 1,5
           do I5G4 = 1,5
       Dself=Dself+(CF11ZL(I5G3,I5G4)*CF11ZL(I5G4,I5G3)+
     &  CF11ZR(I5G3,I5G4)*CF11ZR(I5G4,I5G3))/3d0
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       Dself=Dself+(ChhZ(I6G3,I6G4)**2+6*CsdmsdpZ(I6G3,I6G4)*
     &  CsdmsdpZ(I6G4,I6G3)+2*CsemsepZ(I6G3,I6G4)*CsemsepZ(I6G4,I6G3)+
     &  6*CsupsumZ(I6G3,I6G4)*CsupsumZ(I6G4,I6G3))/6d0
           enddo
           enddo
           do I9G3 = 1,9
           do I9G4 = 1,9
       Dself=Dself+(CLFvFvVZ(I9G3,I9G4)*CLFvFvVZ(I9G4,I9G3)+
     &  CRFvFvVZ(I9G3,I9G4)*CRFvFvVZ(I9G4,I9G3))/3d0
           enddo
           enddo
       Dselfz=Dself*fac

      Dself=0D0
       Dself=Dself+(-10*e**2)/3d0
           do I2G3 = 1,2
       Dself=Dself+(5*e**2)/3d0
           enddo
           do I3G3 = 1,3
       Dself=Dself+(32*e**2)/9d0
           enddo
           do I6G3 = 1,6
       Dself=Dself+(8*e**2)/9d0
           enddo
       Dselfy=Dself*fac

      Dself=0D0
       Dself=Dself+(-10*CWWZ*e)/3d0
           do I2G3 = 1,2
       Dself=Dself+(e*(CHpHmZ+2*CF12ZL(I2G3,I2G3)+2*CF12ZR(I2G3,I2G3)))/3d0
           enddo
           do I3G3 = 1,3
       Dself=Dself+(2*(CddZL+CddZR+ctautauZL+ctautauZR-2*CuuZL-2*CuuZR)*e)/3d0
           enddo
           do I6G3 = 1,6
       Dself=Dself+(e*(CsdmsdpZ(I6G3,I6G3)+CsemsepZ(I6G3,I6G3)-2*
     &  CsupsumZ(I6G3,I6G3)))/3d0
           enddo
       Dselfyz=Dself*fac


      Dself=0D0

       Dself=Dself+(-10*(CWWZ**2+e**2))/3d0
           do I3G3 = 1,3
           do I3G4 = 1,3
       Dself=Dself+2*CbtWm(I3G3,I3G4)*CtbWp(I3G3,I3G4)
           enddo
           enddo
           do I2G3 = 1,2
           do I5G3 = 1,5
       Dself=Dself+(2*(CLF12F11W(I2G3,I5G3)*DCONJG(CLF12F11W(I2G3,I5G3)
     &  )+CRF12F11W(I2G3,I5G3)*DCONJG(CRF12F11W(I2G3,I5G3))))/3d0
           enddo
           enddo
           do I183 = 1,18
           do I6G3 = 1,6
       Dself=Dself+(CSemSvVWp(I6G3,I183)*CSvSepVWm(I183,I6G3))/3d0
           enddo
           enddo
           do I2G3 = 1,2
           do I6G3 = 1,6
       Dself=Dself-(ChHmWp(I2G3,I6G3)*ChHpWm(I6G3,I2G3))/3d0
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       Dself=Dself+CsdsuW(I6G4,I6G3)*DCONJG(CsdsuW(I6G4,I6G3))
           enddo
           enddo
           do I3G3 = 1,3
           do I9G3 = 1,9
       Dself=Dself+(2*(CLFepFvVWm(I3G3,I9G3)*CLFvFemVWp(I9G3,I3G3)+
     &  CRFepFvVWm(I3G3,I9G3)*CRFvFemVWp(I9G3,I3G3)))/3d0
           enddo
           enddo
       Dselfw=Dself*fac


       self=0D0
       self=(-2*CWWZ**2*(3*MW2+5*P2))/3d0+self
           do I183 = 1,18
       self=self+(CSvSvVZVZ(I183,I183)*MSv182(I183))/2d0
           enddo
           do I183 = 1,18
           do I184 = 1,18
       self=self-(CSvSvVZ(I183,I184)**2*(P2-3*MSv182(I183)-3*MSv182(I184)))/6d0
           enddo
           enddo
           do I2G3 = 1,2
       self=self+((CHpHmZ**2*P2)/3d0+2*CHmWpZ(I2G3)**2+(-2*CHpHmZ**2+
     &  CHpHmZZ(I2G3,I2G3))*MassHpm2(I2G3))
           enddo
           do I2G3 = 1,2
           do I2G4 = 1,2
       self=self+(CF12ZL(I2G3,I2G4)*(6*CF12ZR(I2G4,I2G3)*MCha(I2G3)*
     &  MCha(I2G4)+CF12ZL(I2G4,I2G3)*(2*P2-3*MCha2(I2G3)-3*MCha2(I2G4))
     &  )+CF12ZR(I2G3,I2G4)*(6*CF12ZL(I2G4,I2G3)*MCha(I2G3)*MCha(I2G4)+
     &  CF12ZR(I2G4,I2G3)*(2*P2-3*MCha2(I2G3)-3*MCha2(I2G4))))/3d0
           enddo
           enddo
           do I3G3 = 1,3
       self=self+(2*(3*CddZL**2*P2+3*CddZR**2*P2+ctautauZL**2*P2+
     &  ctautauZR**2*P2+3*CuuZL**2*P2+3*CuuZR**2*P2-9*(CddZL-CddZR)**2*
     &  MFd2(I3G3)-3*(ctautauZL-ctautauZR)**2*MFe2(I3G3)-9*CuuZL**2*
     &  MFu2(I3G3)+18*CuuZL*CuuZR*MFu2(I3G3)-9*CuuZR**2*MFu2(I3G3)))
     &  /3d0
           enddo
           do I5G3 = 1,5
           do I5G4 = 1,5
       self=self+(CF11ZL(I5G3,I5G4)*(6*CF11ZR(I5G4,I5G3)*MNeu(I5G3)*
     &  MNeu(I5G4)+CF11ZL(I5G4,I5G3)*(2*P2-3*MNeu2(I5G3)-3*MNeu2(I5G4))
     &  )+CF11ZR(I5G3,I5G4)*(6*CF11ZL(I5G4,I5G3)*MNeu(I5G3)*MNeu(I5G4)+
     &  CF11ZR(I5G4,I5G3)*(2*P2-3*MNeu2(I5G3)-3*MNeu2(I5G4))))/6d0
           enddo
           enddo
           do I6G3 = 1,6
       self=self+(-ChZZ(I6G3)**2-(IC*ChhZZ(I6G3,I6G3)*Masshh2(I6G3))
     &  /2d0+3*CSdSdZZ(I6G3,I6G3)*MSd2(I6G3)+CSeSeZZ(I6G3,I6G3)*
     &  MSe2(I6G3)+3*CSuSuZZ(I6G3,I6G3)*MSu2(I6G3))
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       self=self+(ChhZ(I6G3,I6G4)**2*(P2-3*Masshh2(I6G3)-3*
     &  Masshh2(I6G4))+2*(3*CsdmsdpZ(I6G3,I6G4)*CsdmsdpZ(I6G4,I6G3)*
     &  (P2-3*MSd2(I6G3)-3*MSd2(I6G4))+CsemsepZ(I6G3,I6G4)*
     &  CsemsepZ(I6G4,I6G3)*(P2-3*MSe2(I6G3)-3*MSe2(I6G4))+3*
     &  CsupsumZ(I6G3,I6G4)*CsupsumZ(I6G4,I6G3)*(P2-3*MSu2(I6G3)-3*
     &  MSu2(I6G4))))/6d0
           enddo
           enddo
           do I9G3 = 1,9
           do I9G4 = 1,9
       self=self+(CLFvFvVZ(I9G3,I9G4)*(6*CRFvFvVZ(I9G4,I9G3)*mv9(I9G3)*
     &  mv9(I9G4)+CLFvFvVZ(I9G4,I9G3)*(2*P2-3*mv92(I9G3)-3*mv92(I9G4)))
     &  +CRFvFvVZ(I9G3,I9G4)*(6*CLFvFvVZ(I9G4,I9G3)*mv9(I9G3)*mv9(I9G4)
     &  +CRFvFvVZ(I9G4,I9G3)*(2*P2-3*mv92(I9G3)-3*mv92(I9G4))))/6d0
           enddo
           enddo
       selfz=self*fac 

       self=0D0
       self=(-2*e**2*(3*MW2+5*P2))/3d0+self
           do I2G3 = 1,2
       self=self+((5*e**2*P2)/3d0+2*CHmWpA(I2G3)**2)
           enddo
           do I3G3 = 1,3
       self=(32*e**2*P2)/9d0+self
           enddo
           do I6G3 = 1,6
       self=(8*e**2*P2)/9d0+self
           enddo
       selfy=self*fac 



       self=0D0
       self=((-2*CWWZ*e*(12*MW2+5*P2))/3d0+3*g2**2*MW2*S2W)+self
           do I2G3 = 1,2
       self=self+(CHpHmZ*CW*e*P2*SW+2*CW*e*P2*SW*CF12ZL(I2G3,I2G3)+2*
     &  CW*e*P2*SW*CF12ZR(I2G3,I2G3)-6*CW*SW*CHmWpA(I2G3)*CHmWpZ(I2G3)-
     &  3*e**2*MassHpm2(I2G3)+6*CW2*e**2*MassHpm2(I2G3)-6*CHpHmZ*CW*e*
     &  SW*MassHpm2(I2G3))/(3d0*CW*SW)
           enddo
           do I3G3 = 1,3
       self=(2*(CddZL+CddZR+ctautauZL+ctautauZR-2*CuuZL-2*CuuZR)*e*P2)/3d0+self
           enddo
           do I6G3 = 1,6
       self=self+(e*(CsdmsdpZ(I6G3,I6G3)*(P2-6*MSd2(I6G3))+
     &  CsemsepZ(I6G3,I6G3)*(P2-6*MSe2(I6G3))-2*CsupsumZ(I6G3,I6G3)*
     &  (P2-6*MSu2(I6G3))+(e*(MSd2(I6G3)*(2*CW2+DCONJG(ZD(I6G3,1))*
     &  ZD(I6G3,1)-2*DCONJG(ZD(I6G3,2))*ZD(I6G3,2)+
     &  DCONJG(ZD(I6G3,3))*ZD(I6G3,3)-2*DCONJG(ZD(I6G3,4))*
     &  ZD(I6G3,4)+DCONJG(ZD(I6G3,5))*ZD(I6G3,5)-2*
     &  DCONJG(ZD(I6G3,6))*ZD(I6G3,6))+MSe2(I6G3)*(6*CW2-3*
     &  DCONJG(ZE(I6G3,1))*ZE(I6G3,1)-6*DCONJG(ZE(I6G3,2))*
     &  ZE(I6G3,2)-3*DCONJG(ZE(I6G3,3))*ZE(I6G3,3)-6*
     &  DCONJG(ZE(I6G3,4))*ZE(I6G3,4)-3*DCONJG(ZE(I6G3,5))*
     &  ZE(I6G3,5)-6*DCONJG(ZE(I6G3,6))*ZE(I6G3,6))+2*MSu2(I6G3)*(4*
     &  CW2-DCONJG(ZU(I6G3,1))*ZU(I6G3,1)-4*DCONJG(ZU(I6G3,2))*
     &  ZU(I6G3,2)-DCONJG(ZU(I6G3,3))*ZU(I6G3,3)-4*
     &  DCONJG(ZU(I6G3,4))*ZU(I6G3,4)-DCONJG(ZU(I6G3,5))*ZU(I6G3,5)
     &  -4*DCONJG(ZU(I6G3,6))*ZU(I6G3,6))))/(CW*SW)))/3d0
           enddo
       selfyz=self*fac

       self=0D0

       self=(3*g2**2*MW2-(2*e**2*(6*MW2+5*P2))/3d0-(CWWZ**2*(12*MW2+3*
     &  MZ2+10*P2))/3d0)+self
           do I183 = 1,18
       self=self+(CSvSvVWpVWm(I183,I183)*MSv182(I183))/2d0
           enddo
           do I2G3 = 1,2
       self=self+(CHmWpA(I2G3)**2+CHmWpZ(I2G3)**2+CHmHpWW(I2G3,I2G3)*
     &  MassHpm2(I2G3))
           enddo
           do I3G3 = 1,3
           do I3G4 = 1,3
       self=self+CbtWm(I3G3,I3G4)*CtbWp(I3G3,I3G4)*(2*P2-3*MFd2(I3G4)-
     &  3*MFu2(I3G3))
           enddo
           enddo
           do I2G3 = 1,2
           do I5G3 = 1,5
       self=self+(CLF12F11W(I2G3,I5G3)*(6*DCONJG(CRF12F11W(I2G3,I5G3))*
     &  MCha(I2G3)*MNeu(I5G3)+DCONJG(CLF12F11W(I2G3,I5G3))*(2*P2-3*
     &  MCha2(I2G3)-3*MNeu2(I5G3)))+CRF12F11W(I2G3,I5G3)*(6*
     &  DCONJG(CLF12F11W(I2G3,I5G3))*MCha(I2G3)*MNeu(I5G3)+
     &  DCONJG(CRF12F11W(I2G3,I5G3))*(2*P2-3*MCha2(I2G3)-3*MNeu2(I5G3))
     &  ))/3d0
           enddo
           enddo
           do I6G3 = 1,6
       self=self+(-ChWmWp(I6G3)**2-(IC*ChhWmWp(I6G3,I6G3)*Masshh2(I6G3)
     &  )/2d0+3*CSdSdWW(I6G3,I6G3)*MSd2(I6G3)+CSeSeWW(I6G3,I6G3)*
     &  MSe2(I6G3)+3*CSuSuWW(I6G3,I6G3)*MSu2(I6G3))
           enddo
           do I183 = 1,18
           do I6G3 = 1,6
       self=self+(CSemSvVWp(I6G3,I183)*CSvSepVWm(I183,I6G3)*(P2-3*
     &  MSe2(I6G3)-3*MSv182(I183)))/3d0
           enddo
           enddo
           do I2G3 = 1,2
           do I6G3 = 1,6
       self=self-(ChHmWp(I2G3,I6G3)*ChHpWm(I6G3,I2G3)*(P2-3*
     &  Masshh2(I6G3)-3*MassHpm2(I2G3)))/3d0
           enddo
           enddo
           do I6G3 = 1,6
           do I6G4 = 1,6
       self=self+CsdsuW(I6G4,I6G3)*DCONJG(CsdsuW(I6G4,I6G3))*(P2-3*
     &  MSd2(I6G4)-3*MSu2(I6G3))
           enddo
           enddo
           do I3G3 = 1,3
           do I9G3 = 1,9
       self=self+(CLFepFvVWm(I3G3,I9G3)*(6*CRFvFemVWp(I9G3,I3G3)*
     &  MassFe(I3G3)*mv9(I9G3)+CLFvFemVWp(I9G3,I3G3)*(2*P2-3*MFe2(I3G3)
     &  -3*mv92(I9G3)))+CRFepFvVWm(I3G3,I9G3)*(6*CLFvFemVWp(I9G3,I3G3)*
     &  MassFe(I3G3)*mv9(I9G3)+CRFvFemVWp(I9G3,I3G3)*(2*P2-3*MFe2(I3G3)
     &  -3*mv92(I9G3))))/3d0
           enddo
           enddo
       selfw=self*fac

c       print*, "selfyz",selfw/MW2,selfz/MZ2

       selfQQ= selfy/e**2
       self3Q=(cw*selfyz*sw)/e**2 + (selfy*sw**2)/e**2
       self11=(selfw*sw**2)/e**2
       self33=(cw**2*selfz*sw**2)/e**2 + 
     -    (2*cw*selfyz*sw**3)/e**2 + (selfy*sw**4)/e**2

       dselfQQ= dselfy/e**2
       dself3Q=(cw*dselfyz*sw)/e**2 + (dselfy*sw**2)/e**2
       dself11=(dselfw*sw**2)/e**2
       dself33=(cw**2*dselfz*sw**2)/e**2 + 
     -    (2*cw*dselfyz*sw**3)/e**2 + (dselfy*sw**4)/e**2


       S= 16*pi*(dself33 -dself3Q)
       T= 4*pi/sw**2/cw**2/MZ2*(self11 -self33)
       U = 16*pi*(dself11 -dself33)

c       print*, "S,T,U= ",S,T,U
      end

